@@ -19,12 +19,12 @@ import sys
 import typing
 from contextlib import contextmanager
 from copy import copy
-from functools import lru_cache, reduce, partial
+from functools import lru_cache, reduce, partial, wraps
 from itertools import tee, groupby
 from types import ModuleType
-from typing import (
-    cast, Any, Callable, Dict, Generator, Iterable, List, Mapping, Optional, Set, Tuple,
-    Type, TypeVar, Union,
+from typing import (  # noqa: F401
+    cast, Any, Callable, Dict, Generator, Iterable, List, Mapping, NewType,
+    Optional, Set, Tuple, Type, TypeVar, Union,
 )
 from warnings import warn
 
@@ -49,7 +49,7 @@ _UNKNOWN_MODULE = '?'
 
 T = TypeVar('T', 'Module', 'Class', 'Function', 'Variable')
 
-__pdoc__ = {}  # type: Dict[str, Union[bool, str]]
+__pdoc__: Dict[str, Union[bool, str]] = {}
 
 tpl_lookup = TemplateLookup(
     cache_args=dict(cached=True,
@@ -66,15 +66,6 @@ if os.getenv("XDG_CONFIG_HOME"):
     tpl_lookup.directories.insert(0, path.join(os.getenv("XDG_CONFIG_HOME", ''), "pdoc"))
 
 
-# A surrogate so that the check in Module._link_inheritance()
-# "__pdoc__-overriden key {!r} does not exist" can pick the object up
-# (and not warn).
-# If you know how to keep the warning, but skip the object creation
-# altogether, please make it happen!
-class _BLACKLISTED_DUMMY:
-    pass
-
-
 class Context(dict):
     """
     The context object that maps all documented identifiers
@@ -89,6 +80,13 @@ class Context(dict):
     """
     __pdoc__['Context.__init__'] = False
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        # A surrogate so that the check in Module._link_inheritance()
+        # "__pdoc__-overriden key {!r} does not exist" can see the object
+        # (and not warn).
+        self.blacklisted = getattr(args[0], 'blacklisted', set()) if args else set()
+
 
 _global_context = Context()
 
@@ -128,7 +126,7 @@ def _get_config(**kwargs):
                   | {'module', 'modules', 'http_server', 'external_links', 'search_query'})
     invalid_keys = {k: v for k, v in kwargs.items() if k not in known_keys}
     if invalid_keys:
-        warn('Unknown configuration variables (not in config.mako): {}'.format(invalid_keys))
+        warn(f'Unknown configuration variables (not in config.mako): {invalid_keys}')
     config.update(kwargs)
 
     if 'search_query' in config:
@@ -150,10 +148,9 @@ def _render_template(template_name, **kwargs):
     try:
         t = tpl_lookup.get_template(template_name)
     except TopLevelLookupException:
-        raise OSError(
-            "No template found at any of: {}".format(
-                ', '.join(path.join(p, template_name.lstrip("/"))
-                          for p in tpl_lookup.directories)))
+        paths = [path.join(p, template_name.lstrip('/')) for p in tpl_lookup.directories]
+        raise OSError(f"No template found at any of: {', '.join(paths)}")
+
     try:
         return t.render(**config).strip()
     except Exception:
@@ -224,14 +221,17 @@ def import_module(module: Union[str, ModuleType],
             try:
                 module = importlib.import_module(module_path)
             except Exception as e:
-                raise ImportError('Error importing {!r}: {}: {}'
-                                  .format(module, e.__class__.__name__, e))
+                raise ImportError(f'Error importing {module!r}: {e.__class__.__name__}: {e}')
 
     assert inspect.ismodule(module)
     # If this is pdoc itself, return without reloading. Otherwise later
     # `isinstance(..., pdoc.Doc)` calls won't work correctly.
     if reload and not module.__name__.startswith(__name__):
         module = importlib.reload(module)
+        # We recursively reload all submodules, in case __all_ is used - cf. issue #264
+        for mod_key, mod in list(sys.modules.items()):
+            if mod_key.startswith(module.__name__):
+                importlib.reload(mod)
     return module
 
 
@@ -246,7 +246,7 @@ def _pep224_docstrings(doc_obj: Union['Module', 'Class'], *,
                        _init_tree=None) -> Tuple[Dict[str, str],
                                                  Dict[str, str]]:
     """
-    Extracts PEP-224 docstrings for variables of `doc_obj`
+    Extracts PEP-224 docstrings and doc-comments (`#: ...`) for variables of `doc_obj`
     (either a `pdoc.Module` or `pdoc.Class`).
 
     Returns a tuple of two dicts mapping variable names to their docstrings.
@@ -257,8 +257,8 @@ def _pep224_docstrings(doc_obj: Union['Module', 'Class'], *,
     if isinstance(doc_obj, Module) and doc_obj.is_namespace:
         return {}, {}
 
-    vars = {}  # type: Dict[str, str]
-    instance_vars = {}  # type: Dict[str, str]
+    vars: Dict[str, str] = {}
+    instance_vars: Dict[str, str] = {}
 
     if _init_tree:
         tree = _init_tree
@@ -272,7 +272,7 @@ def _pep224_docstrings(doc_obj: Union['Module', 'Class'], *,
             # Don't emit a warning for builtins that don't have source available
             is_builtin = getattr(doc_obj.obj, '__module__', None) == 'builtins'
             if not is_builtin:
-                warn("Couldn't read PEP-224 variable docstrings from {!r}: {}".format(doc_obj, exc),
+                warn(f"Couldn't read PEP-224 variable docstrings from {doc_obj!r}: {exc}",
                      stacklevel=3 + int(isinstance(doc_obj, Class)))
             return {}, {}
 
@@ -286,27 +286,16 @@ def _pep224_docstrings(doc_obj: Union['Module', 'Class'], *,
                     instance_vars, _ = _pep224_docstrings(doc_obj, _init_tree=node)
                     break
 
-    try:
-        ast_AnnAssign = ast.AnnAssign   # type: Type
-    except AttributeError:  # Python < 3.6
-        ast_AnnAssign = type(None)
-    ast_Assignments = (ast.Assign, ast_AnnAssign)
-
-    for assign_node, str_node in _pairwise(ast.iter_child_nodes(tree)):
-        if not (isinstance(assign_node, ast_Assignments) and
-                isinstance(str_node, ast.Expr) and
-                isinstance(str_node.value, ast.Str)):
-            continue
-
+    def get_name(assign_node):
         if isinstance(assign_node, ast.Assign) and len(assign_node.targets) == 1:
             target = assign_node.targets[0]
-        elif isinstance(assign_node, ast_AnnAssign):
+        elif isinstance(assign_node, ast.AnnAssign):
             target = assign_node.target
             # Skip the annotation. PEP 526 says:
             # > Putting the instance variable annotations together in the class
             # > makes it easier to find them, and helps a first-time reader of the code.
         else:
-            continue
+            return None
 
         if not _init_tree and isinstance(target, ast.Name):
             name = target.id
@@ -316,9 +305,22 @@ def _pep224_docstrings(doc_obj: Union['Module', 'Class'], *,
               target.value.id == 'self'):
             name = target.attr
         else:
-            continue
+            return None
 
         if not _is_public(name) and not _is_whitelisted(name, doc_obj):
+            return None
+
+        return name
+
+    # For handling PEP-224 docstrings for variables
+    for assign_node, str_node in _pairwise(ast.iter_child_nodes(tree)):
+        if not (isinstance(assign_node, (ast.Assign, ast.AnnAssign)) and
+                isinstance(str_node, ast.Expr) and
+                isinstance(str_node.value, ast.Str)):
+            continue
+
+        name = get_name(assign_node)
+        if not name:
             continue
 
         docstring = inspect.cleandoc(str_node.value.s).strip()
@@ -327,6 +329,43 @@ def _pep224_docstrings(doc_obj: Union['Module', 'Class'], *,
 
         vars[name] = docstring
 
+    # For handling '#:' docstrings for variables
+    for assign_node in ast.iter_child_nodes(tree):
+        if not isinstance(assign_node, (ast.Assign, ast.AnnAssign)):
+            continue
+
+        name = get_name(assign_node)
+        if not name:
+            continue
+
+        # Already documented. PEP-224 method above takes precedence.
+        if name in vars:
+            continue
+
+        def get_indent(line):
+            return len(line) - len(line.lstrip())
+
+        source_lines = doc_obj.source.splitlines()  # type: ignore
+        assign_line = source_lines[assign_node.lineno - 1]
+        assign_indent = get_indent(assign_line)
+        comment_lines = []
+        MARKER = '#: '
+        for line in reversed(source_lines[:assign_node.lineno - 1]):
+            if get_indent(line) == assign_indent and line.lstrip().startswith(MARKER):
+                comment_lines.append(line.split(MARKER, maxsplit=1)[1])
+            else:
+                break
+
+        # Since we went 'up' need to reverse lines to be in correct order
+        comment_lines = comment_lines[::-1]
+
+        # Finally: check for a '#: ' comment at the end of the assignment line itself.
+        if MARKER in assign_line:
+            comment_lines.append(assign_line.rsplit(MARKER, maxsplit=1)[1])
+
+        if comment_lines:
+            vars[name] = '\n'.join(comment_lines)
+
     return vars, instance_vars
 
 
@@ -336,8 +375,8 @@ def _is_whitelisted(name: str, doc_obj: Union['Module', 'Class']):
     Returns `True` if `name` (relative or absolute refname) is
     contained in some module's __pdoc__ with a truish value.
     """
-    refname = doc_obj.refname + '.' + name
-    module = doc_obj.module
+    refname = f'{doc_obj.refname}.{name}'
+    module: Optional[Module] = doc_obj.module
     while module:
         qualname = refname[len(module.refname) + 1:]
         if module.__pdoc__.get(qualname) or module.__pdoc__.get(refname):
@@ -352,8 +391,8 @@ def _is_blacklisted(name: str, doc_obj: Union['Module', 'Class']):
     Returns `True` if `name` (relative or absolute refname) is
     contained in some module's __pdoc__ with value False.
     """
-    refname = doc_obj.refname + '.' + name
-    module = doc_obj.module
+    refname = f'{doc_obj.refname}.{name}'
+    module: Optional[Module] = doc_obj.module
     while module:
         qualname = refname[len(module.refname) + 1:]
         if module.__pdoc__.get(qualname) is False or module.__pdoc__.get(refname) is False:
@@ -409,7 +448,7 @@ def _toposort(graph: Mapping[T, Set[T]]) -> Generator[T, None, None]:
         yield from ordered
         if not ordered:
             break
-    assert not graph, "A cyclic dependency exists amongst %r" % graph
+    assert not graph, f"A cyclic dependency exists amongst {graph!r}"
 
 
 def link_inheritance(context: Context = None):
@@ -452,7 +491,7 @@ class Doc:
     """
     __slots__ = ('module', 'name', 'obj', 'docstring', 'inherits')
 
-    def __init__(self, name, module, obj, docstring=None):
+    def __init__(self, name: str, module, obj, docstring: str = None):
         """
         Initializes a documentation object, where `name` is the public
         identifier name, `module` is a `pdoc.Module` object where raw
@@ -485,14 +524,14 @@ class Doc:
         directives resolved (i.e. content included).
         """
 
-        self.inherits = None  # type: Optional[Union[Class, Function, Variable]]
+        self.inherits: Optional[Union[Class, Function, Variable]] = None
         """
         The Doc object (Class, Function, or Variable) this object inherits from,
         if any.
         """
 
     def __repr__(self):
-        return '<{} {!r}>'.format(self.__class__.__name__, self.refname)
+        return f'<{self.__class__.__name__} {self.refname!r}>'
 
     @property  # type: ignore
     @lru_cache()
@@ -548,7 +587,7 @@ class Doc:
             return link_prefix + self._url()
 
         if self.module.name == relative_to.name:
-            return '#' + self.refname
+            return f'#{self.refname}'
 
         # Otherwise, compute relative path from current module to link target
         url = os.path.relpath(self._url(), relative_to.url()).replace(path.sep, '/')
@@ -558,7 +597,7 @@ class Doc:
         return url
 
     def _url(self):
-        return self.module._url() + '#' + self.refname
+        return f'{self.module._url()}#{self.refname}'
 
     def _inherits_top(self):
         """
@@ -617,13 +656,15 @@ class Module(Doc):
         A lookup table for ALL doc objects of all modules that share this context,
         mainly used in `Module.find_ident()`.
         """
+        assert isinstance(self._context, Context), \
+            'pdoc.Module(context=) should be a pdoc.Context instance'
 
         self.supermodule = supermodule
         """
         The parent `pdoc.Module` this module is a submodule of, or `None`.
         """
 
-        self.doc = {}  # type: Dict[str, Union[Module, Class, Function, Variable]]
+        self.doc: Dict[str, Union[Module, Class, Function, Variable]] = {}
         """A mapping from identifier name to a documentation object."""
 
         self._is_inheritance_linked = False
@@ -634,14 +675,14 @@ class Module(Doc):
         var_docstrings, _ = _pep224_docstrings(self)
 
         # Populate self.doc with this module's public members
+        public_objs = []
         if hasattr(self.obj, '__all__'):
-            public_objs = []
             for name in self.obj.__all__:
                 try:
                     obj = getattr(self.obj, name)
                 except AttributeError:
-                    warn("Module {!r} doesn't contain identifier `{}` "
-                         "exported in `__all__`".format(self.module, name))
+                    warn(f"Module {self.module!r} doesn't contain identifier `{name}` "
+                         "exported in `__all__`")
                 if not _is_blacklisted(name, self):
                     obj = inspect.unwrap(obj)
                 public_objs.append((name, obj))
@@ -650,20 +691,25 @@ class Module(Doc):
                 mod = inspect.getmodule(inspect.unwrap(obj))
                 return mod is None or mod.__name__ == self.obj.__name__
 
-            public_objs = [(name, (_BLACKLISTED_DUMMY
-                                   if _is_blacklisted(name, self) else
-                                   inspect.unwrap(obj)))
-                           for name, obj in inspect.getmembers(self.obj)
-                           if ((_is_public(name) or _is_whitelisted(name, self)) and
-                               (_is_blacklisted(name, self) or  # skips unwrapping that follows
-                                is_from_this_module(obj) or name in var_docstrings))]
+            for name, obj in inspect.getmembers(self.obj):
+                if ((_is_public(name) or
+                     _is_whitelisted(name, self)) and
+                        (_is_blacklisted(name, self) or  # skips unwrapping that follows
+                         is_from_this_module(obj) or
+                         name in var_docstrings)):
+
+                    if _is_blacklisted(name, self):
+                        self._context.blacklisted.add(f'{self.refname}.{name}')
+                        continue
+
+                    obj = inspect.unwrap(obj)
+                    public_objs.append((name, obj))
+
             index = list(self.obj.__dict__).index
             public_objs.sort(key=lambda i: index(i[0]))
 
         for name, obj in public_objs:
-            if obj is _BLACKLISTED_DUMMY:
-                self.doc[name] = Variable(name, self, 'dummy', obj=obj)
-            elif _is_function(obj):
+            if _is_function(obj):
                 self.doc[name] = Function(name, self, obj)
             elif inspect.isclass(obj):
                 self.doc[name] = Class(name, self, obj)
@@ -703,7 +749,7 @@ class Module(Doc):
                     continue
 
                 assert self.refname == self.name
-                fullname = "%s.%s" % (self.name, root)
+                fullname = f"{self.name}.{root}"
                 try:
                     m = Module(import_module(fullname),
                                docfilter=docfilter, supermodule=self,
@@ -745,7 +791,7 @@ class Module(Doc):
     __pdoc__['Module.ImportWarning'] = False
 
     @property
-    def __pdoc__(self):
+    def __pdoc__(self) -> dict:
         """This module's __pdoc__ dict, or an empty dict if none."""
         return getattr(self.obj, '__pdoc__', {})
 
@@ -767,20 +813,22 @@ class Module(Doc):
             if docstring is True:
                 continue
 
-            refname = "%s.%s" % (self.refname, name)
+            refname = f"{self.refname}.{name}"
             if docstring in (False, None):
                 if docstring is None:
                     warn('Setting `__pdoc__[key] = None` is deprecated; '
                          'use `__pdoc__[key] = False` '
-                         '(key: {!r}, module: {!r}).'.format(name, self.name))
+                         f'(key: {name!r}, module: {self.name!r}).')
 
                 if name in self._skipped_submodules:
                     continue
 
                 if (not name.endswith('.__init__') and
-                        name not in self.doc and refname not in self._context):
-                    warn('__pdoc__-overriden key {!r} does not exist '
-                         'in module {!r}'.format(name, self.name))
+                        name not in self.doc and
+                        refname not in self._context and
+                        refname not in self._context.blacklisted):
+                    warn(f'__pdoc__-overriden key {name!r} does not exist '
+                         f'in module {self.name!r}')
 
                 obj = self.find_ident(name)
                 cls = getattr(obj, 'cls', None)
@@ -800,8 +848,8 @@ class Module(Doc):
             if isinstance(dobj, External):
                 continue
             if not isinstance(docstring, str):
-                raise ValueError('__pdoc__ dict values must be strings;'
-                                 '__pdoc__[{!r}] is of type {}'.format(name, type(docstring)))
+                raise ValueError('__pdoc__ dict values must be strings; '
+                                 f'__pdoc__[{name!r}] is of type {type(docstring)}')
             dobj.docstring = inspect.cleandoc(docstring)
 
         # Now after docstrings are set correctly, continue the
@@ -862,7 +910,7 @@ class Module(Doc):
         # XXX: Is this corrent? Does it always match
         # `Class.module.name + Class.qualname`?. Especially now?
         # If not, see what was here before.
-        return self.find_ident((cls.__module__ or _UNKNOWN_MODULE) + '.' + cls.__qualname__)
+        return self.find_ident(f'{cls.__module__ or _UNKNOWN_MODULE}.{cls.__qualname__}')
 
     def find_ident(self, name: str) -> Doc:
         """
@@ -881,7 +929,7 @@ class Module(Doc):
 
         return (self.doc.get(_name) or
                 self._context.get(_name) or
-                self._context.get(self.name + '.' + _name) or
+                self._context.get(f'{self.name}.{_name}') or
                 External(name))
 
     def _filter_doc_objs(self, type: Type[T], sort=True) -> List[T]:
@@ -959,32 +1007,39 @@ class Class(Doc):
     """
     __slots__ = ('doc', '_super_members')
 
-    def __init__(self, name, module, obj, *, docstring=None):
+    def __init__(self, name: str, module: Module, obj, *, docstring: str = None):
         assert inspect.isclass(obj)
 
         if docstring is None:
             init_doc = inspect.getdoc(obj.__init__) or ''
             if init_doc == object.__init__.__doc__:
                 init_doc = ''
-            docstring = ((inspect.getdoc(obj) or '') + '\n\n' + init_doc).strip()
+            docstring = f'{inspect.getdoc(obj) or ""}\n\n{init_doc}'.strip()
 
         super().__init__(name, module, obj, docstring=docstring)
 
-        self.doc = {}  # type: Dict[str, Union[Function, Variable]]
+        self.doc: Dict[str, Union[Function, Variable]] = {}
         """A mapping from identifier name to a `pdoc.Doc` objects."""
 
         # Annotations for filtering.
         # Use only own, non-inherited annotations (the rest will be inherited)
         annotations = getattr(self.obj, '__annotations__', {})
 
-        public_objs = [(_name, (_BLACKLISTED_DUMMY
-                                if _is_blacklisted(_name, self) else
-                                inspect.unwrap(obj)))
-                       for _name, obj in _getmembers_all(self.obj)
-                       # Filter only *own* members. The rest are inherited
-                       # in Class._fill_inheritance()
-                       if (_name in self.obj.__dict__ or _name in annotations)
-                       and (_is_public(_name) or _is_whitelisted(_name, self))]
+        public_objs = []
+        for _name, obj in _getmembers_all(self.obj):
+            # Filter only *own* members. The rest are inherited
+            # in Class._fill_inheritance()
+            if ((_name in self.obj.__dict__ or
+                 _name in annotations) and
+                    (_is_public(_name) or
+                     _is_whitelisted(_name, self))):
+
+                if _is_blacklisted(_name, self):
+                    self.module._context.blacklisted.add(f'{self.refname}.{_name}')
+                    continue
+
+                obj = inspect.unwrap(obj)
+                public_objs.append((_name, obj))
 
         def definition_order_index(
                 name,
@@ -1005,9 +1060,7 @@ class Class(Doc):
 
         # Convert the public Python objects to documentation objects.
         for name, obj in public_objs:
-            if obj is _BLACKLISTED_DUMMY:
-                self.doc[name] = Variable(name, self.module, 'dummy', obj=obj, cls=self)
-            elif _is_function(obj):
+            if _is_function(obj):
                 self.doc[name] = Function(
                     name, self.module, obj, cls=self)
             else:
@@ -1044,11 +1097,11 @@ class Class(Doc):
                 if isinstance(c.__dict__[name], staticmethod):
                     return staticmethod
                 return None
-        raise RuntimeError("{}.{} not found".format(cls, name))
+        raise RuntimeError(f"{cls}.{name} not found")
 
     @property
     def refname(self) -> str:
-        return self.module.name + '.' + self.qualname
+        return f'{self.module.name}.{self.qualname}'
 
     def mro(self, only_documented=False) -> List['Class']:
         """
@@ -1058,7 +1111,7 @@ class Class(Doc):
         The list will contain objects of type `pdoc.Class`
         if the types are documented, and `pdoc.External` otherwise.
         """
-        classes = [self.module.find_class(c)
+        classes = [cast(Class, self.module.find_class(c))
                    for c in inspect.getmro(self.obj)
                    if c not in (self.obj, object)]
         if self in classes:
@@ -1081,7 +1134,7 @@ class Class(Doc):
         The objects in the list are of type `pdoc.Class` if available,
         and `pdoc.External` otherwise.
         """
-        return sorted(self.module.find_class(c)
+        return sorted(cast(Class, self.module.find_class(c))
                       for c in type.__subclasses__(self.obj))
 
     def params(self, *, annotate=False, link=None) -> List[str]:
@@ -1199,22 +1252,60 @@ class Class(Doc):
         del self._super_members
 
 
+def maybe_lru_cache(func):
+    cached_func = lru_cache()(func)
+
+    @wraps(func)
+    def wrapper(*args):
+        try:
+            return cached_func(*args)
+        except TypeError:
+            return func(*args)
+
+    return wrapper
+
+
+@maybe_lru_cache
 def _formatannotation(annot):
     """
-    Format annotation, properly handling NewType types
+    Format typing annotation with better handling of `typing.NewType`,
+    `typing.Optional`, `nptyping.NDArray` and other types.
 
-    >>> import typing
-    >>> _formatannotation(typing.NewType('MyType', str))
+    >>> _formatannotation(NewType('MyType', str))
     'MyType'
+    >>> _formatannotation(Optional[Tuple[Optional[int], None]])
+    'Optional[Tuple[Optional[int], None]]'
     """
-    module = getattr(annot, '__module__', '')
-    is_newtype = (getattr(annot, '__qualname__', '').startswith('NewType.') and
-                  module == 'typing')
-    if is_newtype:
-        return annot.__name__
-    if module.startswith('nptyping'):  # GH-231
-        return repr(annot)
-    return inspect.formatannotation(annot)
+    class force_repr(str):
+        __repr__ = str.__str__
+
+    def maybe_replace_reprs(a):
+        # NoneType -> None
+        if a is type(None):  # noqa: E721
+            return force_repr('None')
+        # Union[T, None] -> Optional[T]
+        if (getattr(a, '__origin__', None) is typing.Union and
+                len(a.__args__) == 2 and
+                type(None) in a.__args__):
+            t = inspect.formatannotation(
+                maybe_replace_reprs(next(filter(None, a.__args__))))
+            return force_repr(f'Optional[{t}]')
+        # typing.NewType('T', foo) -> T
+        module = getattr(a, '__module__', '')
+        if module == 'typing' and getattr(a, '__qualname__', '').startswith('NewType.'):
+            return force_repr(a.__name__)
+        # nptyping.types._ndarray.NDArray -> NDArray[(Any,), Int[64]]  # GH-231
+        if module.startswith('nptyping.'):
+            return force_repr(repr(a))
+        # Recurse into typing.Callable/etc. args
+        if hasattr(a, 'copy_with') and hasattr(a, '__args__'):
+            if a is typing.Callable:
+                # Bug on Python < 3.9, https://bugs.python.org/issue42195
+                return a
+            a = a.copy_with(tuple([maybe_replace_reprs(arg) for arg in a.__args__]))
+        return a
+
+    return str(inspect.formatannotation(maybe_replace_reprs(annot)))
 
 
 class Function(Doc):
@@ -1223,7 +1314,7 @@ class Function(Doc):
     """
     __slots__ = ('cls',)
 
-    def __init__(self, name, module, obj, *, cls: Class = None):
+    def __init__(self, name: str, module: Module, obj, *, cls: Class = None):
         """
         Same as `pdoc.Doc`, except `obj` must be a
         Python function object. The docstring is gathered automatically.
@@ -1307,7 +1398,7 @@ class Function(Doc):
         else:
             # Don't warn on variables. The annotation just isn't available.
             if not isinstance(self, Variable):
-                warn("Error handling return annotation for {!r}".format(self), stacklevel=3)
+                warn(f"Error handling return annotation for {self!r}", stacklevel=3)
 
         if annot is inspect.Parameter.empty or not annot:
             return ''
@@ -1316,7 +1407,7 @@ class Function(Doc):
             s = annot
         else:
             s = _formatannotation(annot)
-            s = re.sub(r'\b(typing\.)?ForwardRef\((?P<quot>[\"\'])(?P<str>.*?)(?P=quot)\)',
+            s = re.sub(r'\bForwardRef\((?P<quot>[\"\'])(?P<str>.*?)(?P=quot)\)',
                        r'\g<str>', s)
         s = s.replace(' ', '\N{NBSP}')  # Better line breaks in html signatures
 
@@ -1373,7 +1464,7 @@ class Function(Doc):
                 if isinstance(value, enum.Enum):
                     replacement = str(value)
                 elif inspect.isclass(value):
-                    replacement = (value.__module__ or _UNKNOWN_MODULE) + '.' + value.__qualname__
+                    replacement = f'{value.__module__ or _UNKNOWN_MODULE}.{value.__qualname__}'
                 elif ' at 0x' in repr(value):
                     replacement = re.sub(r' at 0x\w+', '', repr(value))
 
@@ -1427,16 +1518,16 @@ class Function(Doc):
                     annotation = annotation.strip("'")
                 if link:
                     annotation = re.sub(r'[\w\.]+', _linkify, annotation)
-                formatted += ':\N{NBSP}' + annotation
+                formatted += f':\N{NBSP}{annotation}'
             if p.default is not EMPTY:
                 if p.annotation is not EMPTY:
-                    formatted += '\N{NBSP}=\N{NBSP}' + repr(p.default)
+                    formatted += f'\N{NBSP}=\N{NBSP}{repr(p.default)}'
                 else:
-                    formatted += '=' + repr(p.default)
+                    formatted += f'={repr(p.default)}'
             if p.kind == p.VAR_POSITIONAL:
-                formatted = '*' + formatted
+                formatted = f'*{formatted}'
             elif p.kind == p.VAR_KEYWORD:
-                formatted = '**' + formatted
+                formatted = f'**{formatted}'
 
             params.append(formatted)
 
@@ -1469,10 +1560,10 @@ class Function(Doc):
                 # See: https://github.com/pdoc3/pdoc/pull/148#discussion_r407114141
                 module_basename = self.module.name.rsplit('.', maxsplit=1)[-1]
                 if module_basename in string and module_basename not in _globals:
-                    string = re.sub(r'(?<!\.)\b{}\.\b'.format(module_basename), '', string)
+                    string = re.sub(fr'(?<!\.)\b{module_basename}\.\b', '', string)
 
                 try:
-                    exec('def {}: pass'.format(string), _globals, _locals)
+                    exec(f'def {string}: pass', _globals, _locals)
                 except SyntaxError:
                     continue
                 signature = inspect.signature(_locals[self.name])
@@ -1484,7 +1575,7 @@ class Function(Doc):
 
     @property
     def refname(self) -> str:
-        return (self.cls.refname if self.cls else self.module.refname) + '.' + self.name
+        return f'{self.cls.refname if self.cls else self.module.refname}.{self.name}'
 
 
 class Variable(Doc):
@@ -1494,8 +1585,8 @@ class Variable(Doc):
     """
     __slots__ = ('cls', 'instance_var')
 
-    def __init__(self, name, module, docstring, *,
-                 obj=None, cls: Class = None, instance_var=False):
+    def __init__(self, name: str, module: Module, docstring, *,
+                 obj=None, cls: Class = None, instance_var: bool = False):
         """
         Same as `pdoc.Doc`, except `cls` should be provided
         as a `pdoc.Class` object when this is a class or instance
@@ -1518,12 +1609,12 @@ class Variable(Doc):
     @property
     def qualname(self) -> str:
         if self.cls:
-            return self.cls.qualname + '.' + self.name
+            return f'{self.cls.qualname}.{self.name}'
         return self.name
 
     @property
     def refname(self) -> str:
-        return (self.cls.refname if self.cls else self.module.refname) + '.' + self.name
+        return f'{self.cls.refname if self.cls else self.module.refname}.{self.name}'
 
     def type_annotation(self, *, link=None) -> str:
         """Formatted variable type annotation or empty string if none."""
@@ -1554,7 +1645,7 @@ class External(Doc):
         form.
         """
 
-    def __init__(self, name):
+    def __init__(self, name: str):
         """
         Initializes an external identifier with `name`, where `name`
         should be a fully qualified name.
@@ -1565,4 +1656,4 @@ class External(Doc):
         """
         `External` objects return absolute urls matching `/{name}.ext`.
         """
-        return '/%s.ext' % self.name
+        return f'/{self.name}.ext'
