@@ -27,7 +27,7 @@ aa = parser.add_argument
 mode_aa = parser.add_mutually_exclusive_group().add_argument
 
 aa(
-    '--version', action='version', version='%(prog)s ' + pdoc.__version__)
+    '--version', action='version', version=f'%(prog)s {pdoc.__version__}')
 aa(
     "modules",
     type=str,
@@ -129,8 +129,8 @@ DEFAULT_HOST, DEFAULT_PORT = 'localhost', 8080
 def _check_host_port(s):
     if s and ':' not in s:
         raise argparse.ArgumentTypeError(
-            "'{}' doesn't match '[HOST]:[PORT]'. "
-            "Specify `--http :` to use default hostname and port.".format(s))
+            f"'{s}' doesn't match '[HOST]:[PORT]'. "
+            "Specify `--http :` to use default hostname and port.")
     return s
 
 
@@ -141,7 +141,7 @@ aa(
     metavar='HOST:PORT',
     help="When set, pdoc will run as an HTTP server providing documentation "
          "for specified modules. If you just want to use the default hostname "
-         "and port ({}:{}), set the parameter to :.".format(DEFAULT_HOST, DEFAULT_PORT),
+         f"and port ({DEFAULT_HOST}:{DEFAULT_PORT}), set the parameter to :.",
 )
 aa(
     "--skip-errors",
@@ -212,14 +212,14 @@ class _WebDoc(BaseHTTPRequestHandler):
             import_path = self.path[:-4].lstrip("/")
             resolved = self.resolve_ext(import_path)
             if resolved is None:  # Try to generate the HTML...
-                print("Generating HTML for %s on the fly..." % import_path, file=sys.stderr)
+                print(f"Generating HTML for {import_path} on the fly...", file=sys.stderr)
                 try:
                     out = pdoc.html(import_path.split(".")[0], **self.template_config)
                 except Exception as e:
-                    print('Error generating docs: {}'.format(e), file=sys.stderr)
+                    print(f'Error generating docs: {e}', file=sys.stderr)
                     # All hope is lost.
                     code = 404
-                    out = "External identifier <code>%s</code> not found." % import_path
+                    out = f"External identifier <code>{import_path}</code> not found."
             else:
                 return self.redirect(resolved)
         # Redirect '/pdoc' to '/pdoc/' so that relative links work
@@ -236,8 +236,11 @@ class _WebDoc(BaseHTTPRequestHandler):
                 import traceback
                 from html import escape
                 code = 404
-                out = "Error importing module <code>{}</code>:\n\n<pre>{}</pre>".format(
-                    self.import_path_from_req_url, escape(traceback.format_exc()))
+                out = (
+                    "Error importing module "
+                    f"<code>{self.import_path_from_req_url}</code>:\n\n"
+                    f"<pre>{escape(traceback.format_exc())}</pre>"
+                )
                 out = out.replace('\n', '<br>')
 
         self.send_response(code)
@@ -262,7 +265,7 @@ class _WebDoc(BaseHTTPRequestHandler):
         """
         return pdoc.html(self.import_path_from_req_url,
                          reload=True, http_server=True, external_links=True,
-                         skip_errors=args.skip_errors,
+                         skip_errors=self.args.skip_errors,
                          **self.template_config)
 
     def resolve_ext(self, import_path):
@@ -282,7 +285,7 @@ class _WebDoc(BaseHTTPRequestHandler):
             p = path.join(*parts[0:i])
             realp = exists(p)
             if realp is not None:
-                return "/%s#%s" % (realp.lstrip("/"), import_path)
+                return f"/{realp.lstrip('/')}#{import_path}"
         return None
 
     @property
@@ -312,8 +315,7 @@ def _quit_if_exists(m: pdoc.Module, ext: str):
 
     for pth in paths:
         if path.lexists(pth):
-            print("File '%s' already exists. Delete it, or run with --force" % pth,
-                  file=sys.stderr)
+            print(f"File '{pth}' already exists. Delete it, or run with --force", file=sys.stderr)
             sys.exit(1)
 
 
@@ -362,9 +364,9 @@ def _print_pdf(modules, **kwargs):
 
 
 def _warn_deprecated(option, alternative='', use_config_mako=False):
-    msg = 'Program option `{}` is deprecated.'.format(option)
+    msg = f'Program option `{option}` is deprecated.'
     if alternative:
-        msg += ' Use `' + alternative + '`'
+        msg += f' Use `{alternative}`'
         if use_config_mako:
             msg += ' or override config.mako template'
         msg += '.'
@@ -406,8 +408,8 @@ def _generate_lunr_search(modules: List[pdoc.Module],
             url_cache[url] = len(url_cache)
         return url_cache[url]
 
-    index = []  # type: List[Dict]
-    url_cache = {}  # type: Dict[str, int]
+    index: List[Dict] = []
+    url_cache: Dict[str, int] = {}
     for top_module in modules:
         recursive_add_to_index(top_module)
     urls = sorted(url_cache.keys(), key=url_cache.__getitem__)
@@ -430,7 +432,9 @@ def main(_args=None):
     global args
     args = _args or parser.parse_args()
 
-    warnings.simplefilter("once", DeprecationWarning)
+    # If warnings not externally managed, show deprecation warnings
+    if not sys.warnoptions:
+        warnings.simplefilter("once", DeprecationWarning)
 
     if args.close_stdin:
         sys.stdin.close()
@@ -453,9 +457,8 @@ def main(_args=None):
             template_config[key] = value
         except Exception:
             raise ValueError(
-                'Error evaluating --config statement "{}". '
+                f'Error evaluating --config statement "{config_str}". '
                 'Make sure string values are quoted?'
-                .format(config_str)
             )
 
     if args.html_no_source:
@@ -470,8 +473,7 @@ def main(_args=None):
 
     if args.template_dir is not None:
         if not path.isdir(args.template_dir):
-            print('Error: Template dir {!r} is not a directory'.format(args.template_dir),
-                  file=sys.stderr)
+            print(f'Error: Template dir {args.template_dir!r} is not a directory', file=sys.stderr)
             sys.exit(1)
         pdoc.tpl_lookup.directories.insert(0, args.template_dir)
 
@@ -495,7 +497,7 @@ def main(_args=None):
                 with open(pth) as f:
                     sys.path.append(path.join(libdir, f.readline().rstrip()))
             except IOError:
-                warn('Invalid egg-link in venv: {!r}'.format(pth))
+                warn(f'Invalid egg-link in venv: {pth!r}')
 
     if args.http:
         template_config['link_prefix'] = "/"
@@ -508,9 +510,9 @@ def main(_args=None):
         host = host or DEFAULT_HOST
         port = int(port or DEFAULT_PORT)
 
-        print('Starting pdoc server on {}:{}'.format(host, port), file=sys.stderr)
+        print(f'Starting pdoc server on {host}:{port}', file=sys.stderr)
         httpd = HTTPServer((host, port), _WebDoc)
-        print("pdoc server ready at http://%s:%d" % (host, port), file=sys.stderr)
+        print(f"pdoc server ready at http://{host}:{port}", file=sys.stderr)
 
         # Allow tests to perform `pdoc.cli._httpd.shutdown()`
         global _httpd
@@ -537,7 +539,8 @@ def main(_args=None):
     if args.pdf:
         _print_pdf(modules, **template_config)
         import textwrap
-        print("""
+        PANDOC_CMD = textwrap.indent(_PANDOC_COMMAND, '    ')
+        print(f"""
 PDF-ready markdown written to standard output.
                               ^^^^^^^^^^^^^^^
 Convert this file to PDF using e.g. Pandoc:
@@ -562,7 +565,7 @@ or using Python-Markdown and Chrome/Chromium/WkHtmlToPDF:
 
     wkhtmltopdf --encoding utf8 -s A4 --print-media-type pdf.html pdf.pdf
 
-or similar, at your own discretion.""".format(PANDOC_CMD=textwrap.indent(_PANDOC_COMMAND, '    ')),
+or similar, at your own discretion.""",
               file=sys.stderr)
         sys.exit(0)
 
@@ -578,10 +581,11 @@ or similar, at your own discretion.""".format(PANDOC_CMD=textwrap.indent(_PANDOC
             # Two blank lines between two modules' texts
             sys.stdout.write(os.linesep * (1 + 2 * int(module != modules[-1])))
 
-    lunr_config = pdoc._get_config(**template_config).get('lunr_search')
-    if lunr_config is not None:
-        _generate_lunr_search(
-            modules, lunr_config.get("index_docstrings", True), template_config)
+    if args.html:
+        lunr_config = pdoc._get_config(**template_config).get('lunr_search')
+        if lunr_config is not None:
+            _generate_lunr_search(
+                modules, lunr_config.get("index_docstrings", True), template_config)
 
 
 _PANDOC_COMMAND = '''\
