@@ -1,6 +1,7 @@
 """
 Unit tests for pdoc package.
 """
+import doctest
 import enum
 import inspect
 import os
@@ -63,7 +64,7 @@ def chdir(path):
         os.chdir(old)
 
 
-def run(*args, _check=True, **kwargs) -> int:
+def run(*args, **kwargs) -> int:
     params = (('--' + key.replace('_', '-'), value)
               for key, value in kwargs.items())
     params = list(filter(None, chain.from_iterable(params)))  # type: ignore
@@ -106,29 +107,30 @@ class CliTest(unittest.TestCase):
     """
     ALL_FILES = [
         'example_pkg',
-        'example_pkg/index.html',
-        'example_pkg/index.m.html',
-        'example_pkg/module.html',
-        'example_pkg/_private',
-        'example_pkg/_private/index.html',
-        'example_pkg/_private/module.html',
-        'example_pkg/subpkg',
-        'example_pkg/subpkg/_private.html',
-        'example_pkg/subpkg/index.html',
-        'example_pkg/subpkg2',
-        'example_pkg/subpkg2/_private.html',
-        'example_pkg/subpkg2/module.html',
-        'example_pkg/subpkg2/index.html',
+        os.path.join('example_pkg', 'index.html'),
+        os.path.join('example_pkg', 'index.m.html'),
+        os.path.join('example_pkg', 'module.html'),
+        os.path.join('example_pkg', '_private'),
+        os.path.join('example_pkg', '_private', 'index.html'),
+        os.path.join('example_pkg', '_private', 'module.html'),
+        os.path.join('example_pkg', 'subpkg'),
+        os.path.join('example_pkg', 'subpkg', '_private.html'),
+        os.path.join('example_pkg', 'subpkg', 'index.html'),
+        os.path.join('example_pkg', 'subpkg2'),
+        os.path.join('example_pkg', 'subpkg2', '_private.html'),
+        os.path.join('example_pkg', 'subpkg2', 'module.html'),
+        os.path.join('example_pkg', 'subpkg2', 'index.html'),
     ]
-    PUBLIC_FILES = [f for f in ALL_FILES if '/_' not in f]
-
-    if os.name == 'nt':
-        ALL_FILES = [i.replace('/', '\\') for i in ALL_FILES]
-        PUBLIC_FILES = [i.replace('/', '\\') for i in PUBLIC_FILES]
+    PUBLIC_FILES = [f for f in ALL_FILES if (os.path.sep + '_') not in f]
 
     def setUp(self):
         pdoc.reset()
 
+    @unittest.skipIf(sys.version_info < (3, 7), 'pdoc._formatannotation fails on Py3.6')
+    def test_project_doctests(self):
+        doctests = doctest.testmod(pdoc)
+        assert not doctests.failed and doctests.attempted, doctests
+
     def _basic_html_assertions(self, expected_files=PUBLIC_FILES):
         # Output directory tree layout is as expected
         files = glob('**', recursive=True)
@@ -190,7 +192,7 @@ class CliTest(unittest.TestCase):
             '.subpkg2': [f for f in self.PUBLIC_FILES
                          if 'subpkg2' in f or f == EXAMPLE_MODULE],
             '._private': [f for f in self.ALL_FILES
-                          if EXAMPLE_MODULE + '/_private' in f or f == EXAMPLE_MODULE],
+                          if EXAMPLE_MODULE + os.path.sep + '_private' in f or f == EXAMPLE_MODULE],
         }
         for package, expected_files in package_files.items():
             with self.subTest(package=package):
@@ -203,7 +205,8 @@ class CliTest(unittest.TestCase):
         filenames_files = {
             ('module.py',): ['module.html'],
             ('module.py', 'subpkg2'): ['module.html', 'subpkg2',
-                                       'subpkg2/index.html', 'subpkg2/module.html'],
+                                       os.path.join('subpkg2', 'index.html'),
+                                       os.path.join('subpkg2', 'module.html')],
         }
         with chdir(TESTS_BASEDIR):
             for filenames, expected_files in filenames_files.items():
@@ -216,9 +219,11 @@ class CliTest(unittest.TestCase):
 
     def test_html_multiple_files(self):
         with chdir(TESTS_BASEDIR):
-            with run_html(EXAMPLE_MODULE + '/module.py', EXAMPLE_MODULE + '/subpkg2'):
-                self._basic_html_assertions(
-                    ['module.html', 'subpkg2', 'subpkg2/index.html', 'subpkg2/module.html'])
+            with run_html(os.path.join(EXAMPLE_MODULE, 'module.py'),
+                          os.path.join(EXAMPLE_MODULE, 'subpkg2')):
+                self._basic_html_assertions(['module.html', 'subpkg2',
+                                             os.path.join('subpkg2', 'index.html'),
+                                             os.path.join('subpkg2', 'module.html')])
 
     def test_html_identifier(self):
         for package in ('', '._private'):
@@ -227,12 +232,12 @@ class CliTest(unittest.TestCase):
                 with run_html(EXAMPLE_MODULE + package, filter='A',
                               config='show_source_code=False'):
                     self._check_files(['A'], ['CONST', 'B docstring'])
-        self.assertIn('__pdoc__', cm.warning.args[0])
+        self.assertIn('Code reference `example_pkg.B`', cm.warning.args[0])
 
     def test_html_ref_links(self):
         with run_html(EXAMPLE_MODULE, config='show_source_code=False'):
             self._check_files(
-                file_pattern=EXAMPLE_MODULE + '/index.html',
+                file_pattern=os.path.join(EXAMPLE_MODULE, 'index.html'),
                 include_patterns=[
                     'href="#example_pkg.B">',
                     'href="#example_pkg.A">',
@@ -269,11 +274,15 @@ class CliTest(unittest.TestCase):
                               file_pattern='example_pkg/module.html')
             self._check_files(include_patterns=["'../../doc-search.html#'"],
                               file_pattern='example_pkg/subpkg/index.html')
+        # Only build lunr search when --html
+        with redirect_streams() as (_, stderr):
+            run(EXAMPLE_MODULE,  config='lunr_search={"fuzziness": 1}')
+            self.assertFalse(stderr.read())
 
     def test_force(self):
         with run_html(EXAMPLE_MODULE):
             with redirect_streams() as (stdout, stderr):
-                returncode = run(EXAMPLE_MODULE, _check=False, html=None, output_dir=os.getcwd())
+                returncode = run(EXAMPLE_MODULE, html=None, output_dir=os.getcwd())
                 self.assertNotEqual(returncode, 0)
                 self.assertNotEqual(stderr.getvalue(), '')
 
@@ -355,8 +364,7 @@ class CliTest(unittest.TestCase):
                 run(EXAMPLE_MODULE, config='show_type_annotations=False')
                 out = stdout.getvalue()
 
-            header = 'Module {}\n{:=<{}}'.format(EXAMPLE_MODULE, '',
-                                                 len('Module ') + len(EXAMPLE_MODULE))
+            header = f"Module {EXAMPLE_MODULE}\n{'':=<{len('Module ') + len(EXAMPLE_MODULE)}}"
             self.assertIn(header, out)
             for pattern in include_patterns:
                 self.assertIn(pattern, out)
@@ -371,7 +379,7 @@ class CliTest(unittest.TestCase):
                         run(*(os.path.join(EXAMPLE_MODULE, f) for f in files))
                         out = stdout.getvalue()
                     for f in files:
-                        header = 'Module {}\n'.format(os.path.splitext(f)[0])
+                        header = f'Module {os.path.splitext(f)[0]}\n'
                         self.assertIn(header, out)
 
     def test_text_identifier(self):
@@ -390,10 +398,7 @@ class CliTest(unittest.TestCase):
             err = stderr.getvalue()
         self.assertIn('pdoc3.github.io', out)
         self.assertIn('pandoc', err)
-
-        pdoc_Doc_params = str(inspect.signature(pdoc.Doc.__init__)).replace('self, ', '')
-        self.assertIn(pdoc_Doc_params.replace(' ', ''),
-                      out.replace('>', '').replace('\n', '').replace(' ', ''))
+        self.assertIn('Doc(name', out.replace('>', '').replace('\n', '').replace(' ', ''))
 
     @unittest.skipUnless('PDOC_TEST_PANDOC' in os.environ, 'PDOC_TEST_PANDOC not set/requested')
     def test_pdf_pandoc(self):
@@ -472,7 +477,7 @@ class ApiTest(unittest.TestCase):
             for module, (name_suffix, submodules) in modules.items():
                 with self.subTest(module=module):
                     m = pdoc.Module(module)
-                    self.assertEqual(repr(m), "<Module '{}'>".format(m.obj.__name__))
+                    self.assertEqual(repr(m), f"<Module '{m.obj.__name__}'>")
                     self.assertEqual(m.name, EXAMPLE_MODULE + name_suffix)
                     self.assertEqual(sorted(m.name for m in m.submodules()),
                                      [EXAMPLE_MODULE + '.' + m for m in submodules])
@@ -584,12 +589,14 @@ class ApiTest(unittest.TestCase):
     def test__pdoc__dict(self):
         module = pdoc.import_module(EXAMPLE_MODULE)
         with patch.object(module, '__pdoc__', {'B': False}):
+            pdoc.reset()
             mod = pdoc.Module(module)
             pdoc.link_inheritance()
             self.assertIn('A', mod.doc)
             self.assertNotIn('B', mod.doc)
 
         with patch.object(module, '__pdoc__', {'B.f': False}):
+            pdoc.reset()
             mod = pdoc.Module(module)
             pdoc.link_inheritance()
             self.assertIn('B', mod.doc)
@@ -598,13 +605,23 @@ class ApiTest(unittest.TestCase):
 
         # GH-125: https://github.com/pdoc3/pdoc/issues/125
         with patch.object(module, '__pdoc__', {'B.inherited': False}):
+            pdoc.reset()
             mod = pdoc.Module(module)
             pdoc.link_inheritance()
             self.assertNotIn('inherited', mod.doc['B'].doc)
 
+        # Ensure "overridden key doesn't exist" warning is raised
+        with patch.object(module, '__pdoc__', {'xxx': False}):
+            pdoc.reset()
+            mod = pdoc.Module(module)
+            with self.assertWarns(UserWarning) as cm:
+                pdoc.link_inheritance()
+            self.assertIn("'xxx' does not exist", cm.warning.args[0])
+
         # GH-99: https://github.com/pdoc3/pdoc/issues/99
         module = pdoc.import_module(EXAMPLE_MODULE + '._exclude_dir')
         with patch.object(module, '__pdoc__', {'downloaded_modules': False}, create=True):
+            pdoc.reset()
             mod = pdoc.Module(module)
             # GH-206: https://github.com/pdoc3/pdoc/issues/206
             with warnings.catch_warnings(record=True) as cm:
@@ -787,10 +804,6 @@ class ApiTest(unittest.TestCase):
             pdoc.link_inheritance()
         self.assertFalse(w)
 
-        mod._is_inheritance_linked = False
-        with self.assertWarns(UserWarning):
-            pdoc.link_inheritance()
-
         # Test inheritance across modules
         pdoc.reset()
         mod = pdoc.Module(EXAMPLE_MODULE + '._test_linking')
@@ -805,7 +818,7 @@ class ApiTest(unittest.TestCase):
         self.assertNotEqual(b.inherits, a)
 
     def test_context(self):
-        context = {}
+        context = pdoc.Context()
         pdoc.Module(pdoc, context=context)
         self.assertIn('pdoc', context)
         self.assertIn('pdoc.cli', context)
@@ -863,7 +876,7 @@ class ApiTest(unittest.TestCase):
 
         # typed, linked
         def link(dobj):
-            return '<a href="{}">{}</a>'.format(dobj.url(relative_to=mod), dobj.qualname)
+            return f'<a href="{dobj.url(relative_to=mod)}">{dobj.qualname}</a>'
 
         self.assertEqual(func.params(annotate=True, link=link),
                          ['a:\N{NBSP}int', '*b',
@@ -897,13 +910,21 @@ class ApiTest(unittest.TestCase):
             pdoc.Function('bug253', mod, bug253_newtype_annotation).params(annotate=True),
             ['a:\N{NBSP}CustomType'])
 
+        # typing.Callable bug
+        def f(a: typing.Callable):
+            return
+
+        self.assertEqual(
+            pdoc.Function('f', mod, f).params(annotate=True),
+            ['a:\N{NBSP}Callable'])
+
         # builtin callables with signatures in docstrings
         from itertools import repeat
         self.assertEqual(pdoc.Function('repeat', mod, repeat).params(), ['object', 'times'])
         self.assertEqual(pdoc.Function('slice', mod, slice).params(), ['start', 'stop', 'step'])
 
         class get_sample(repeat):
-            """ get_sample(self: pdoc.int, pos: int) -> Tuple[int, float] """
+            """ get_sample(self: int, pos: int) -> Tuple[int, float] """
         self.assertEqual(pdoc.Function('get_sample', mod, get_sample).params(annotate=True),
                          ['self:\xa0int', 'pos:\xa0int'])
         self.assertEqual(pdoc.Function('get_sample', mod, get_sample).return_annotation(),
@@ -932,10 +953,9 @@ class ApiTest(unittest.TestCase):
 
         mod = DUMMY_PDOC_MODULE
         cls = pdoc.Class('Foobar', mod, Foobar)
-        self.assertEqual(cls.doc['prop'].type_annotation(), 'Union[int,\N{NBSP}NoneType]')
+        self.assertEqual(cls.doc['prop'].type_annotation(), 'Optional[int]')
 
     @ignore_warnings
-    @unittest.skipIf(sys.version_info < (3, 6), 'variable annotation unsupported in <Py3.6')
     def test_Variable_type_annotation_py36plus(self):
         with temp_dir() as path:
             filename = os.path.join(path, 'module36syntax.py')
@@ -1050,7 +1070,6 @@ class Foo:
         self.assertEqual(f.url(relative_to=c.module), '#example_pkg.D.overridden')
         self.assertEqual(f.url(top_ancestor=1), 'example_pkg/index.html#example_pkg.B.overridden')
 
-    @unittest.skipIf(sys.version_info < (3, 6), reason="only deterministic on CPython 3.6+")
     def test_sorting(self):
         module = EXAMPLE_PDOC_MODULE
 
@@ -1082,7 +1101,6 @@ class Foo:
         self.assertIn('Module', mod.doc)
 
     @ignore_warnings
-    @unittest.skipIf(sys.version_info < (3, 6), 'variable type annotation unsupported in <Py3.6')
     def test_class_members(self):
         module = DUMMY_PDOC_MODULE
 
@@ -1106,6 +1124,39 @@ class Foo:
         self.assertIsInstance(cls.doc['name'], pdoc.Variable)
         self.assertEqual(cls.doc['name'].type_annotation(), 'str')
 
+    def test_doc_comment_docstrings(self):
+        with temp_dir() as path:
+            filename = os.path.join(path, 'doc_comment_docstrs.py')
+            with open(filename, 'w') as f:
+                f.write('''#: Not included
+
+#: Line 1
+#: Line 2
+var1 = 1  #: Line 3
+
+#: Not included
+var2 = 1
+"""PEP-224 takes precedence"""
+
+#: This should not appear
+class C:
+    #: class var
+    class_var = 1
+
+    #: This also should not show
+    def __init__(self):
+       #: instance var
+       self.instance_var = 1
+''')
+
+            mod = pdoc.Module(pdoc.import_module(filename))
+
+            self.assertEqual(mod.doc['var1'].docstring, 'Line 1\nLine 2\nLine 3')
+            self.assertEqual(mod.doc['var2'].docstring, 'PEP-224 takes precedence')
+            self.assertEqual(mod.doc['C'].docstring, '')
+            self.assertEqual(mod.doc['C'].doc['class_var'].docstring, 'class var')
+            self.assertEqual(mod.doc['C'].doc['instance_var'].docstring, 'instance var')
+
 
 class HtmlHelpersTest(unittest.TestCase):
     """
@@ -1155,7 +1206,7 @@ reference: `package.foo`
         module.doc['_x_x_'] = pdoc.Variable('_x_x_', module, '')
 
         def link(dobj):
-            return '<a href="{}">{}</a>'.format(dobj.url(relative_to=module), dobj.qualname)
+            return f'<a href="{dobj.url(relative_to=module)}">{dobj.qualname}</a>'
 
         html = to_html(text, module=module, link=link)
         self.assertEqual(html, expected)
@@ -1195,7 +1246,7 @@ pdoc
 '''
 
         def link(dobj):
-            return '<a>{}</a>'.format(dobj.qualname)
+            return f'<a>{dobj.qualname}</a>'
 
         html = to_html(text, module=PDOC_PDOC_MODULE, link=link)
         self.assertEqual(html, expected)
@@ -1271,7 +1322,7 @@ class Docformats(unittest.TestCase):
 
     @staticmethod
     def _link(dobj, *args, **kwargs):
-        return '<a>{}</a>'.format(dobj.refname)
+        return f'<a>{dobj.refname}</a>'
 
     def test_numpy(self):
         expected = '''<p>Summary line.</p>
@@ -1614,7 +1665,7 @@ class HttpTest(unittest.TestCase):
             with redirect_streams() as (stdout, stderr):
                 t = threading.Thread(
                     target=cli.main,
-                    args=(cli.parser.parse_args(['--http', ':%d' % port] + modules),))
+                    args=(cli.parser.parse_args(['--http', f':{port}'] + modules),))
                 t.start()
                 sleep(.1)
 
@@ -1623,7 +1674,7 @@ class HttpTest(unittest.TestCase):
                     raise AssertionError
 
                 try:
-                    yield 'http://localhost:{}/'.format(port)
+                    yield f'http://localhost:{port}/'
                 except Exception:
                     sys.__stderr__.write(stderr.getvalue())
                     sys.__stdout__.write(stdout.getvalue())
